# SPDX-License-Identifier: GPL-2.0-or-later

from dasbus.client.proxy import InterfaceProxy, ObjectProxy
from dasbus.connection import MessageBus, SystemMessageBus, SessionMessageBus


class Hirte:
    def __init__(self, bus: MessageBus = None, use_systembus=True) -> None:
        self.use_systembus = use_systembus

        if bus is not None:
            self.bus = bus
        elif self.use_systembus:
            self.bus = SystemMessageBus()
        else:
            self.bus = SessionMessageBus()

    def get_proxy(self) -> InterfaceProxy | ObjectProxy:
        raise Exception("Not implemented!")



# SPDX-License-Identifier: GPL-2.0-or-later
from typing import Callable
from dasbus.typing import *
from dasbus.connection import MessageBus

from pyhirte.gen.consts import *
from pyhirte.gen.hirte import *


class HirteAgent(Hirte):
    def __init__(self, bus: MessageBus = None, use_systembus=True) -> None:
        super().__init__(bus, use_systembus)

        self.agent_proxy = None

    def get_proxy(self) -> InterfaceProxy | ObjectProxy:
        if self.agent_proxy is None:
            self.agent_proxy = self.bus.get_proxy(
                HIRTE_AGENT_DBUS_INTERFACE, HIRTE_OBJECT_PATH
            )

        return self.agent_proxy

    def create_proxy(self, local_service_name: str, node: str, unit: str) -> None:
        self.get_proxy().CreateProxy(
            local_service_name,
            node,
            unit,
        )

    def remove_proxy(self, local_service_name: str, node: str, unit: str) -> None:
        self.get_proxy().RemoveProxy(
            local_service_name,
            node,
            unit,
        )



# SPDX-License-Identifier: GPL-2.0-or-later
from typing import Callable
from dasbus.typing import *
from dasbus.connection import MessageBus

from pyhirte.gen.consts import *
from pyhirte.gen.hirte import *


class HirteMetrics(Hirte):
    def __init__(
        self, metrics_path: ObjPath, bus: MessageBus = None, use_systembus=True
    ) -> None:
        super().__init__(bus, use_systembus)

        self.metrics_path = metrics_path
        self.metrics_proxy = None

    def get_proxy(self) -> InterfaceProxy | ObjectProxy:
        if self.metrics_proxy is None:
            self.metrics_proxy = self.bus.get_proxy(
                HIRTE_DBUS_INTERFACE, self.metrics_path
            )

        return self.metrics_proxy

    def on_start_unit_job_metrics(
        self,
        callback: Callable[
            [
                str,
                str,
                str,
                UInt64,
                UInt64,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().StartUnitJobMetrics.connect(callback)

    def on_agent_job_metrics(
        self,
        callback: Callable[
            [
                str,
                str,
                str,
                UInt64,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().AgentJobMetrics.connect(callback)



# SPDX-License-Identifier: GPL-2.0-or-later
from typing import Callable
from dasbus.typing import *
from dasbus.connection import MessageBus

from pyhirte.gen.consts import *
from pyhirte.gen.hirte import *


class HirteNode(Hirte):
    def __init__(
        self, node_name: str, bus: MessageBus = None, use_systembus=True
    ) -> None:
        super().__init__(bus, use_systembus)

        self.node_name = node_name
        self.node_proxy = None

    def get_proxy(self) -> InterfaceProxy | ObjectProxy:
        if self.node_proxy is None:
            manager = self.bus.get_proxy(HIRTE_DBUS_INTERFACE, HIRTE_OBJECT_PATH)

            node_path = manager.GetNode(self.node_name)
            self.node_proxy = self.bus.get_proxy(HIRTE_DBUS_INTERFACE, node_path)

        return self.node_proxy

    def start_unit(self, name: str, mode: str) -> ObjPath:
        return self.get_proxy().StartUnit(
            name,
            mode,
        )

    def stop_unit(self, name: str, mode: str) -> ObjPath:
        return self.get_proxy().StopUnit(
            name,
            mode,
        )

    def reload_unit(self, name: str, mode: str) -> ObjPath:
        return self.get_proxy().ReloadUnit(
            name,
            mode,
        )

    def restart_unit(self, name: str, mode: str) -> ObjPath:
        return self.get_proxy().RestartUnit(
            name,
            mode,
        )

    def get_unit_properties(self, name: str, interface: str) -> Structure:
        return self.get_proxy().GetUnitProperties(
            name,
            interface,
        )

    def get_unit_property(self, name: str, interface: str, property: str) -> Variant:
        return self.get_proxy().GetUnitProperty(
            name,
            interface,
            property,
        )

    def set_unit_properties(
        self, name: str, runtime: bool, keyvalues: List[Tuple[str, Variant]]
    ) -> None:
        self.get_proxy().SetUnitProperties(
            name,
            runtime,
            keyvalues,
        )

    def enable_unit_files(
        self, files: List[str], runtime: bool, force: bool
    ) -> Tuple[bool, List[Tuple[str, str, str]],]:
        return self.get_proxy().EnableUnitFiles(
            files,
            runtime,
            force,
        )

    def disable_unit_files(
        self, files: List[str], runtime: bool
    ) -> List[Tuple[str, str, str]]:
        return self.get_proxy().DisableUnitFiles(
            files,
            runtime,
        )

    def list_units(
        self,
    ) -> List[Tuple[str, str, str, str, str, str, ObjPath, UInt32, str, ObjPath]]:
        return self.get_proxy().ListUnits()

    def reload(self) -> None:
        self.get_proxy().Reload()

    def get_name(self) -> str:
        self.get_proxy().Name

    def get_status(self) -> str:
        self.get_proxy().Status



# SPDX-License-Identifier: GPL-2.0-or-later
from typing import Callable
from dasbus.typing import *
from dasbus.connection import MessageBus

from pyhirte.gen.consts import *
from pyhirte.gen.hirte import *


class HirteJob(Hirte):
    def __init__(
        self, job_path: ObjPath, bus: MessageBus = None, use_systembus=True
    ) -> None:
        super().__init__(bus, use_systembus)

        self.job_path = job_path
        self.job_proxy = None

    def get_proxy(self) -> InterfaceProxy | ObjectProxy:
        if self.job_proxy is None:
            self.job_proxy = self.bus.get_proxy(HIRTE_DBUS_INTERFACE, self.job_path)

        return self.job_proxy

    def cancel(self) -> None:
        self.get_proxy().Cancel()

    def get_id(self) -> UInt32:
        self.get_proxy().Id

    def get_node(self) -> str:
        self.get_proxy().Node

    def get_unit(self) -> str:
        self.get_proxy().Unit

    def get_job_type(self) -> str:
        self.get_proxy().JobType

    def get_state(self) -> str:
        self.get_proxy().State



from dasbus.connection import MessageBus
from typing import Callable, List, NamedTuple
from collections import namedtuple
from dasbus.loop import EventLoop

from pyhirte import HirteNode, HirteManager
from pyhirte import UInt32, ObjPath


UnitInfo = namedtuple(
    "UnitInfo",
    [
        "name",
        "description",
        "load_state",
        "active_state",
        "sub_state",
        "follower",
        "object_path",
        "job_id",
        "job_type",
        "job_object_path",
    ],
)


UnitChange = namedtuple(
    "UnitChange",
    [
        "change_type",
        "symlink_file",
        "symlink_destination",
    ],
)


class EnableUnitsResponse(NamedTuple):
    carries_install_info: bool
    changes: List[UnitChange]


class HirteUnit:
    def __init__(
        self, node_name: str, bus: MessageBus = None, use_systembus=True
    ) -> None:
        self.node = HirteNode(node_name, bus, use_systembus)
        self.job_result = ""

    def _wait_for_complete(
        self, operation: Callable[[str, str], ObjPath], unit: str
    ) -> str:
        event_loop = EventLoop()

        def on_job_removed(
            _: UInt32, job_path: ObjPath, node_name: str, unit_name: str, result: str
        ) -> None:
            if job_path == wait_for_job_path:
                self.job_result = result
                event_loop.quit()

        HirteManager(bus=self.node.bus).on_job_removed(on_job_removed)

        wait_for_job_path = operation(unit, "replace")
        event_loop.run()

        job_result = self.job_result
        self.job_result = ""
        return job_result

    def start_unit(self, unit: str) -> str:
        return self._wait_for_complete(self.node.start_unit, unit)

    def stop_unit(self, unit: str) -> str:
        return self._wait_for_complete(self.node.stop_unit, unit)

    def restart_unit(self, unit: str) -> str:
        return self._wait_for_complete(self.node.restart_unit, unit)

    def reload_unit(self, unit: str) -> str:
        return self._wait_for_complete(self.node.reload_unit, unit)

    def list_units(self) -> List[UnitInfo]:
        return [UnitInfo(*unit) for unit in self.node.list_units()]

    def enable_unit_files(self, files: List[str]) -> EnableUnitsResponse:
        return self.node.enable_unit_files(files, False, False)

    def disable_unit_files(self, files: List[str]) -> List[UnitChange]:
        return self.node.disable_unit_files(files, False)



# SPDX-License-Identifier: GPL-2.0-or-later
from typing import Callable
from dasbus.typing import *
from dasbus.connection import MessageBus

from pyhirte.gen.consts import *
from pyhirte.gen.hirte import *


class HirteManager(Hirte):
    def __init__(self, bus: MessageBus = None, use_systembus=True) -> None:
        super().__init__(bus, use_systembus)

        self.manager_proxy = None

    def get_proxy(self) -> InterfaceProxy | ObjectProxy:
        if self.manager_proxy is None:
            self.manager_proxy = self.bus.get_proxy(
                HIRTE_DBUS_INTERFACE, HIRTE_OBJECT_PATH
            )

        return self.manager_proxy

    def list_units(
        self,
    ) -> List[Tuple[str, str, str, str, str, str, ObjPath, UInt32, str, ObjPath]]:
        return self.get_proxy().ListUnits()

    def list_nodes(self) -> List[Tuple[str, ObjPath, str]]:
        return self.get_proxy().ListNodes()

    def get_node(self, name: str) -> ObjPath:
        return self.get_proxy().GetNode(
            name,
        )

    def create_monitor(self) -> ObjPath:
        return self.get_proxy().CreateMonitor()

    def enable_metrics(self) -> None:
        self.get_proxy().EnableMetrics()

    def disable_metrics(self) -> None:
        self.get_proxy().DisableMetrics()

    def on_job_new(
        self,
        callback: Callable[
            [
                UInt32,
                ObjPath,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().JobNew.connect(callback)

    def on_job_removed(
        self,
        callback: Callable[
            [
                UInt32,
                ObjPath,
                str,
                str,
                str,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().JobRemoved.connect(callback)

    def on_node_connection_state_changed(
        self,
        callback: Callable[
            [
                str,
                str,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().NodeConnectionStateChanged.connect(callback)

    def get_nodes(self) -> List[str]:
        self.get_proxy().Nodes



# SPDX-License-Identifier: GPL-2.0-or-later

HIRTE_DEFAULT_PORT = 842
HIRTE_DEFAULT_HOST = "127.0.0.1"

HIRTE_DBUS_INTERFACE = "org.containers.hirte"
HIRTE_OBJECT_PATH = "/org/containers/hirte"
HIRTE_AGENT_DBUS_INTERFACE = "org.containers.hirte.Agent"



# SPDX-License-Identifier: GPL-2.0-or-later
from typing import Callable
from dasbus.typing import *
from dasbus.connection import MessageBus

from pyhirte.gen.consts import *
from pyhirte.gen.hirte import *


class HirteMonitor(Hirte):
    def __init__(
        self, monitor_path: ObjPath, bus: MessageBus = None, use_systembus=True
    ) -> None:
        super().__init__(bus, use_systembus)

        self.monitor_path = monitor_path
        self.monitor_proxy = None

    def get_proxy(self) -> InterfaceProxy | ObjectProxy:
        if self.monitor_proxy is None:
            self.monitor_proxy = self.bus.get_proxy(
                HIRTE_DBUS_INTERFACE, self.monitor_path
            )

        return self.monitor_proxy

    def close(self) -> None:
        self.get_proxy().Close()

    def subscribe(self, node: str, unit: str) -> UInt32:
        return self.get_proxy().Subscribe(
            node,
            unit,
        )

    def unsubscribe(self, id: UInt32) -> None:
        self.get_proxy().Unsubscribe(
            id,
        )

    def subscribe_list(self, targets: Tuple[str, List[str]]) -> UInt32:
        return self.get_proxy().SubscribeList(
            targets,
        )

    def on_unit_properties_changed(
        self,
        callback: Callable[
            [
                str,
                str,
                str,
                Structure,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().UnitPropertiesChanged.connect(callback)

    def on_unit_state_changed(
        self,
        callback: Callable[
            [
                str,
                str,
                str,
                str,
                str,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().UnitStateChanged.connect(callback)

    def on_unit_new(
        self,
        callback: Callable[
            [
                str,
                str,
                str,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().UnitNew.connect(callback)

    def on_unit_removed(
        self,
        callback: Callable[
            [
                str,
                str,
                str,
            ],
            None,
        ],
    ) -> None:
        """
        callback:
        """
        self.get_proxy().UnitRemoved.connect(callback)
